
/*
 * Written by Scott Auge scott_auge@yahoo.com sauge@amduus.com
 * Copyright (c) 2001 Amduus Information Works, Inc.  www.amduus.com
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Amduus Information Works
 *      Inc. and its contributors.
 * 4. Neither the name of Amduus Information Works, Inc. nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY AMDUUS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AMDUUS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

DEF VAR RCSVersion AS CHARACTER INIT "$Header: /home/appl/opensrc/dbemail/src/RCS/PopMail.p,v 1.6 2001/12/19 17:51:09 sauge Exp sauge $" NO-UNDO.

/* Retreive email from a pop mail box.  Deposit in the database */

DEF VAR cConfFile         AS CHARACTER NO-UNDO.
DEF VAR cMailServer       AS CHARACTER NO-UNDO.
DEF VAR cUser             AS CHARACTER NO-UNDO.
DEF VAR cPassword         AS CHARACTER NO-UNDO.

DEF VAR iNumberEmails     AS INTEGER NO-UNDO.
DEF VAR iCurrentEmail     AS INTEGER NO-UNDO.

DEF VAR hHandle AS HANDLE NO-UNDO.
DEF VAR iResult           AS INTEGER NO-UNDO.
DEF VAR cMessageText      AS CHARACTER NO-UNDO.

{pop3api.i}
{MakeID2.i}
{emailhdr.i}

{Log.i NEW}

{LogIt.i '"Begin Program"'}

/* Determine our configurations */

ASSIGN cConfFile = OS-GETENV("CONFFILE").

RUN inird.p (INPUT cConfFile,
             INPUT "pop3",
             INPUT "PopServer",
             OUTPUT cMailServer).

{LogIt.i '"MailServer="' cMailServer}

RUN inird.p (INPUT cConfFile,
             INPUT "pop3",
             INPUT "User",
             OUTPUT cUser).

{LogIt.i '"User="' cUser}

RUN inird.p (cConfFile,
             "pop3",
             "Password",
             OUTPUT cPassword).

{LogIt.i '"Password="' cPassword}

/* Connect to the pop server */

RUN ConnectToServer (INPUT cMailServer,
                     OUTPUT hHandle,
                     OUTPUT iResult).

{LogIt.i '"ConnectToServer="' iResult}

RUN Login (INPUT hHandle,
           INPUT cUser,
           INPUT cPassword,
           OUTPUT cMessageText).

{LogIt.i '"Login="' TRIM(cMessageText)}

/* Determine number of emails */

RUN ListMessages (INPUT hHandle,
                  OUTPUT cMessageText).

ASSIGN cMessageText = TRIM(cMessageText).

{LogIt.i '"ListMessages Returns "' cMessageText}

ASSIGN iNumberEmails = NUM-ENTRIES(cMessageText).

{LogIt.i '"Number of Messages: "' iNumberEmails}

/* Retrieve each of these emails if greater than 1 */

DO iCurrentEmail = 1 TO iNumberEmails:

  {LogIt.i '"Retreiving Message "' iCurrentEmail}

  RUN RetrieveMessage (INPUT hHandle,
                       INPUT STRING(iCurrentEmail),
                       OUTPUT cMessageText).

  RUN ipCreateEmailRecord (INPUT TRIM (cMessageText)).

  /* Deletion was in it's own loop, but upon interruption of the program */
  /* all emails previously handled by the loop would be duplicated again. */

  RUN DeleteMessage (INPUT hHandle,
                     INPUT STRING(iCurrentEmail),
                     OUTPUT cMessageText).

  {LogIt.i '"DeleteMessage Responds: "' TRIM(cMessageText)}

END.  /* DO iCurrentEmail = 1 TO iNumberEmails: */

/* Disconnect from the server */

RUN Logout (INPUT hHandle,
            OUTPUT cMessageText).

{LogIt.i '"Logout Responds: "' TRIM(cMessageText)}

RUN DisconnectFromServer (INPUT hHandle).

{LogIt.i '"Disconnected from server"'}
{LogIt.i '"End Program"'}

/*************************** Internal Procedures *****************************/

PROCEDURE ipCreateEmailRecord:

  DEF INPUT PARAMETER cMessageText AS CHARACTER NO-UNDO.

  CREATE Email.

  ASSIGN Email.EmailID = MakeID2(10).

  RUN EmailHeader (cMessageText, "Subject:", OUTPUT Email.Subject).
  RUN EmailHeader (cMessageText, "To:", OUTPUT EMail.ToAddress).
  RUN EmailHeader (cMessageText, "From:", OUTPUT Email.FromAddress).
  IF Email.FromAddress = ? THEN 
    RUN EmailHeader (cMessageText, "Sender:", OUTPUT Email.FromAddress).
  RUN EmailBody (cMessageText, OUTPUT Email.Body).

  /* Something is just fouled up and we lost this one. */

  IF Email.FromAddress = ? AND Email.Subject = ? THEN DELETE Email.

END. /* PROCEDURE ipCreateEmailRecord */

