/*
 * Written by Scott Auge scott_auge@yahoo.com sauge@amduus.com
 * Copyright (c) 2001 Amduus Information Works, Inc.  www.amduus.com
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Amduus Information Works
 *      Inc. and its contributors.
 * 4. Neither the name of Amduus Information Works, Inc. nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY AMDUUS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AMDUUS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

DEF VAR EMAILHDR_I_RCSVersion AS CHARACTER INIT "$Header: /home/appl/opensrc/dbemail/src/RCS/emailhdr.i,v 1.3 2001/12/17 22:38:17 sauge Exp sauge $" NO-UNDO.

/* Various routines, that when given a raw email, will return the header */
/* information.                                                          */

/* WARNING: ATTEMPTED FUNCTIONS BUT RAN INTO STACK ERRORS ON LARGE EMAILS */

PROCEDURE EMailHeader:

  DEF INPUT PARAMETER cRawEmail      AS CHARACTER NO-UNDO.
  DEF INPUT PARAMETER cHeader        AS CHARACTER NO-UNDO.
  DEF OUTPUT PARAMETER cReturnString AS CHARACTER NO-UNDO.

  DEF VAR iNumberLines AS INTEGER NO-UNDO.
  DEF VAR iCurrentLine AS INTEGER NO-UNDO.

  DEF VAR cCurrentLine  AS CHARACTER NO-UNDO.

  ASSIGN iNumberLines = NUM-ENTRIES(cRawEmail, CHR(13)).

  DO iCurrentLine = 1 TO iNumberLines:

    ASSIGN cCurrentLine = TRIM(ENTRY(iCurrentLine, cRawEmail, CHR(13))).

    /* A blank line seperates header from body of the email.  If we encounter */
    /* this, we no longer are in the header and should leave.                 */

    IF TRIM (cCurrentLine) = "" THEN LEAVE.

    /* If the currentline begins the header element, then we want to return */
    /* what is left of the first : in the line.                             */

    IF cCurrentLine BEGINS cHeader THEN DO:

      ASSIGN cReturnString = 
      SUBSTRING(cCurrentLine, INDEX(cCurrentLine, ":") + 1).
      RETURN.

    END. /* IF cCurrentLine BEGINS cHeader */

  END. /* DO iCurrentLine = 1 TO iNumberLines */
 
  /* Email does not have the given header, then return a ? */

  ASSIGN cReturnString = ?.

END. /* PROCEDURE  EmailHeader */

PROCEDURE EmailBody:

  DEF INPUT PARAMETER cRawEmail      AS CHARACTER NO-UNDO.
  DEF OUTPUT PARAMETER cReturnString AS CHARACTER NO-UNDO.

  DEF VAR iNumberLines AS INTEGER NO-UNDO.
  DEF VAR iCurrentLine AS INTEGER NO-UNDO.

  DEF VAR cCurrentLine  AS CHARACTER NO-UNDO.

  DEF VAR lFoundBody    AS LOGICAL NO-UNDO.

  ASSIGN iNumberLines = NUM-ENTRIES(cRawEmail, CHR(13)).
  ASSIGN lFoundBody = NO.
  ASSIGN cReturnString = "".

  DO iCurrentLine = 1 TO iNumberLines:

    ASSIGN cCurrentLine = TRIM(ENTRY(iCurrentLine, cRawEmail, CHR(13))).

    IF TRIM(cCurrentLine) = "" AND lFoundBody = NO THEN DO:
      ASSIGN lFoundBody = YES.
      NEXT.
    END.

    IF lFoundBody = YES THEN 
      ASSIGN cReturnString = cReturnString + cCurrentLine + CHR(13) + CHR(10).

  END. /* DO iCurrentLine = 1 TO iNumberLines */

END. /* PROCEDURE  EmailBody */
